/*
 * Created on 2005/02/23
 * Author aki@www.xucker.jpn.org
 * License Apache2.0 or Common Public License
 */
package org.jpn.xucker.commons.audio;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * 
 *
 */
public class AudioUtils {
    public static Log log=LogFactory.getLog(AudioUtils.class);
    public static long countByteAt(AudioFormat format,long millisecond){
        if(millisecond==0){
            return 0;
        }
        
        return countByteAt(format.getFrameRate(),format.getSampleSizeInBits(),format.getChannels(),millisecond);
    }
    
    public static long countByteAt(double framerate,int samplebit,int channel,long millisecond){
        if(millisecond==0){
            return 0;
        }
        long rateIndex=(long)((framerate/1000*(millisecond)));
        //System.out.println(""+format.getFrameRate()+","+rateIndex);
        return rateIndex*samplebit/8*channel;
    }
    
    
    
    
    public static void appendSilenceWave(OutputStream output,double framerate,int samplebit,int channel,long millisecond){
        try {
            long bt=countByteAt(framerate,samplebit,channel,millisecond);
            
            int buffersize=10240;
            BufferedOutputStream bout=new BufferedOutputStream(output,buffersize);
            byte buff[]=new byte[buffersize];
            
            long num=bt/buffersize;
            int mod=(int)(bt%buffersize);
            for(int i=0;i<num;i++)
                {
                bout.write(buff);
                }
            bout.write(new byte[mod]);
           
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    public static void repeatFile(File inputFile,File outputFile,int repeatTime){
        log.trace("input "+inputFile.getAbsolutePath()+" outputFile "+outputFile+" repeat "+repeatTime);
        BufferedOutputStream bout;
        try {
            bout = new BufferedOutputStream(new FileOutputStream(outputFile));
            byte buffer[]=new byte[10240];
            for(int i=0;i<repeatTime;i++){
                BufferedInputStream binput=new BufferedInputStream(new FileInputStream(inputFile));
                int read;
                while((read=binput.read(buffer))!=-1){
                    bout.write(buffer,0,read);
                }
                binput.close();
                
            }
            bout.close();
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
       
    }
    public static  void repeatWave(File inputFile,File outputFile,int repeatTime,long silentmillisecond,boolean silencelast) throws IOException{

        try {
         
            
            
            File tmpFile=File.createTempFile("appendSilence",".wav");
            log.trace("create tmp:"+tmpFile.getAbsolutePath());
            
            FileOutputStream fout=new FileOutputStream(tmpFile);
            InputStream input = new FileInputStream(inputFile);
            
            
            AudioInputStream audioInputStream=null;
            
                 audioInputStream=AudioSystem.getAudioInputStream(inputFile);
                  AudioFormat audioFormat =audioInputStream.getFormat();
                  
                  
                  
                  
                  
                   int readBytes = 0;
                   byte[] bytedata = new byte[10280];
                   long readedByte=0;
                   while (readBytes != -1) {
                    
                    readBytes = audioInputStream.read(bytedata, 0, bytedata.length);
                     if (readBytes >= 0) {
                        fout.write(bytedata,0,readBytes);
                         
                     }
                   }
                   
                   audioInputStream.close();
                   input.close();
                   fout.close();
                   
                   File repeatTmpFile=File.createTempFile("repeatWave",".wav");
                   log.trace("create tmp:"+repeatTmpFile.getAbsolutePath());
                   FileOutputStream repeatOut=new FileOutputStream(repeatTmpFile);
                   for(int i=0;i<repeatTime;i++){
                       appendFile(tmpFile,repeatOut);
                       
                       //appendSilenceWave(repeatTmpFile,silentmillisecond);
                       if(i!=repeatTime-1 || silencelast==true){
                           appendSilenceWave(repeatOut,audioFormat.getFrameRate(),audioFormat.getSampleSizeInBits(),audioFormat.getChannels(),silentmillisecond);
                           
                       }
                       
                   }
            
                   repeatOut.close();
                   
            
            
            
            
            
            
            AudioInputStream clipInput=new AudioInputStream(new FileInputStream(repeatTmpFile),audioFormat,repeatTmpFile.length()/4);
            
             
            
            FileOutputStream output=new FileOutputStream(outputFile);
            
            int writtenBytes=-1;
            
          
                writtenBytes=AudioSystem.write(
     			        clipInput
     			        ,AudioFileFormat.Type.WAVE,
     				output);
            
            
            log.trace("write:"+writtenBytes);
            
            
            clipInput.close();
             
            output.close();
            boolean result=tmpFile.delete();
            log.trace("delte tmp:"+result);
            
           result=repeatTmpFile.delete();
            log.trace("delte repatTmp:"+result);
        }  catch (UnsupportedAudioFileException e) {
            log.fatal("",e);
        }
              
          }
    /**
     * @param tmpFile
     * @param repeatOut
     */
    private static void appendFile(File tmpFile, FileOutputStream repeatOut) {
        try {
            
            BufferedOutputStream bout=new BufferedOutputStream(repeatOut);
            BufferedInputStream binput=new BufferedInputStream(new FileInputStream(tmpFile));
            int b;
            while((b=binput.read())!=-1){
                bout.write(b);
            }
            binput.close();
           
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * @param tmpFile
     * @param repeatTmpFile
     */
    private static void appendFile(File tmpFile, File repeatTmpFile) {
        try {
            FileOutputStream fout=new FileOutputStream(repeatTmpFile,true);
            BufferedOutputStream bout=new BufferedOutputStream(fout);
            BufferedInputStream binput=new BufferedInputStream(new FileInputStream(tmpFile));
            int b;
            while((b=binput.read())!=-1){
                bout.write(b);
            }
            binput.close();
            bout.close();
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
    }

    public static  void appendSilenceWave(File inputFile,long millisecond) {
        
          try {
           
              
              
              File tmpFile=File.createTempFile("appendSilence",".wav");
              log.trace("create tmp:"+tmpFile.getAbsolutePath());
              
              FileOutputStream fout=new FileOutputStream(tmpFile);
              InputStream input = new FileInputStream(inputFile);
              
              
              AudioInputStream audioInputStream=null;
              
                   audioInputStream=AudioSystem.getAudioInputStream(inputFile);
                    AudioFormat audioFormat =audioInputStream.getFormat();
                    
                    
                    
                    
                    
                     int readBytes = 0;
                     byte[] bytedata = new byte[10280];
                     long readedByte=0;
                     while (readBytes != -1) {
                      
                      readBytes = audioInputStream.read(bytedata, 0, bytedata.length);
                       if (readBytes >= 0) {
                          fout.write(bytedata,0,readBytes);
                           
                       }
                     }
                     
                     audioInputStream.close();
              
                     appendSilenceWave(fout,audioFormat.getFrameRate(),audioFormat.getSampleSizeInBits(),audioFormat.getChannels(),millisecond);
                     
                     fout.close();
              
              
              
              
              
              
              AudioInputStream clipInput=new AudioInputStream(new FileInputStream(tmpFile),audioFormat,tmpFile.length()/4);
              
               
              FileOutputStream output=new FileOutputStream(inputFile);
              
              int writtenBytes=-1;
              
            
                  writtenBytes=AudioSystem.write(
       			        clipInput
       			        ,AudioFileFormat.Type.WAVE,
       				output);
              
              
              log.trace("write:"+writtenBytes);
              
              
              clipInput.close();
               
              output.close();
              boolean result=tmpFile.delete();
              log.trace("delte tmp:"+result);
          } catch (FileNotFoundException e) {
              log.fatal("",e);
          } catch (UnsupportedAudioFileException e) {
              log.fatal("",e);
          } catch (IOException e) {
              log.fatal("",e);
          } catch (Exception e) {
              log.fatal("",e);
          }
          
      }
}
